package org.cocos2dx.javascript;

import java.util.ArrayList;
import java.util.List;

import org.json.JSONException;
import org.json.JSONObject;

import android.app.AlertDialog;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.util.Log;
import android.widget.EditText;
import android.widget.Toast;

import com.masterdangermoral.wander.*;

public class NativeUtils {
	private static AppActivity app = null;

	private static native void CPPJsNotiFun(String msg, String jsonstr);

	public static void setApp(AppActivity curApp) {
		app = curApp;
	}

	// 测试部分...
	public static void showTest() {
		app.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				AlertDialog.Builder builder = null;
				builder = new AlertDialog.Builder(app)
						.setTitle("请选择")
						.setIcon(android.R.drawable.ic_dialog_info)
						.setSingleChoiceItems(
								new String[] { "给xx写信", "分享测试",
										"NativeUtils信息", "浏览器", "显示uuid" }, 0,
								new DialogInterface.OnClickListener() {
									public void onClick(DialogInterface dialog,
											int which) {
										Log.v("single select:",
												String.format("%d", which));

										switch (which) {
										case 0:
											NativeUtils.suggestByMail(
													"zhangyuntao@163.com",
													"title", "content");
											break;
										case 1:
											NativeUtils
													.goSocialActivity("这个太好玩了!");
											break;
										case 2:
											String str = "";
											str = str
													+ NativeUtils.getAppName();
											str += NativeUtils.getCurVersion();
											str += NativeUtils.getDeviceInfo();
											str += NativeUtils.getPackNameStr();
											str += NativeUtils.getChannelStr();
											NativeUtils.askAlert("title", str,
													"no", "yes", -1);
											break;
										case 3:
											NativeUtils
													.showModalWebView("http://www.baidu.com");
											break;
										case 4:
											String uid = NativeUtils
													.getInstallationUUID();
											NativeUtils.okAlert("title", uid,
													"ok", -1);
											break;
										}
										dialog.dismiss();
									}
								}).setNegativeButton("取消", null);
				builder.show();
			}
		});

	}

	public static boolean isOnline() {
		ConnectivityManager cm = (ConnectivityManager) app
				.getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo netInfo = cm.getActiveNetworkInfo();
		return netInfo != null && netInfo.isConnected();
	}

	public static boolean isWifiConnected() {
		Context context = app.getApplicationContext();
		if (context != null) {
			ConnectivityManager mConnectivityManager = (ConnectivityManager) context
					.getSystemService(Context.CONNECTIVITY_SERVICE);
			NetworkInfo mWiFiNetworkInfo = mConnectivityManager
					.getNetworkInfo(ConnectivityManager.TYPE_WIFI);
			if (mWiFiNetworkInfo != null) {
				return mWiFiNetworkInfo.isAvailable();
			}
		}
		return false;
	}

	public static String getInstallationUUID() {
		return Installation.id(app);
	}

	public static boolean isAppInstalled(String packageName) {
		final PackageManager packageManager = app.getPackageManager();// 获取packagemanager
		List<PackageInfo> pinfo = packageManager.getInstalledPackages(0);// 获取所有已安装程序的包信息
		List<String> pName = new ArrayList<String>();// 用于存储所有已安装程序的包名
		// 从pinfo中将包名字逐一取出，压入pName list中
		if (pinfo != null) {
			for (int i = 0; i < pinfo.size(); i++) {
				String pn = pinfo.get(i).packageName;
				pName.add(pn);
			}
		}
		return pName.contains(packageName);// 判断pName中是否有目标程序的包名，有TRUE，没有FALSE
	}

	public static void openInstalledApp(String packageName, String activity) {
		if (NativeUtils.isAppInstalled(packageName)) {
			Intent i = new Intent();
			ComponentName cn = new ComponentName(packageName, activity);
			i.setComponent(cn);
			Log.v("openApp", packageName);
			app.startActivityForResult(i, app.RESULT_OK);
		}
	}

	public static void goToAppStore(final String packageName) {
		app.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				Uri uri = Uri.parse("market://details?id=" + packageName);//
				Intent intent = new Intent(Intent.ACTION_VIEW, uri);
				intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
				app.startActivity(intent);
			}
		});
	}

	public static void goRating() {
		// Log.v("unImp", "没有实现goRating");
		String appPackage = app.getPackageName();
		Log.v("goRating", appPackage);
		NativeUtils.goToAppStore(appPackage);
	}

	public static void goMoreApp(final String company) {
		NativeUtils.goRating();
		// Log.v("unImp", "没有实现goMoreApp" + company);
		// app.runOnUiThread(new Runnable() {
		// @Override
		// public void run() {
		// Intent intent = new Intent(Intent.ACTION_VIEW);
		// intent.setData(Uri.parse("market://search?q=pub:" + company));
		// app.startActivity(intent);
		// }
		// });
	}

	public static void suggestByMail(final String mailto, final String subject,
			final String content) {
		app.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				Intent data = new Intent(Intent.ACTION_SENDTO);
				data.setData(Uri.parse(String.format("mailto:%s", mailto)));
				data.putExtra(Intent.EXTRA_SUBJECT, subject);
				data.putExtra(Intent.EXTRA_TEXT, content);
				app.startActivity(data);
			}
		});
	}

	public static void goSocialActivity(final String content) {
		Log.v("goSocialActivity", "here");
		app.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				Log.v("goSocialActivity", "here");
				Intent intent = new Intent(Intent.ACTION_SEND);

				intent.setType("text/plain");
				intent.putExtra(Intent.EXTRA_SUBJECT, "分享");
				intent.putExtra(Intent.EXTRA_TEXT, content);
				app.startActivity(Intent.createChooser(intent, app.getTitle()));
			}
		});
	}

	public static boolean isSocialAvailable() {
		Log.v("isSocialAvailable", "here");
		return true;
	}

	public static String getAppName() {
		PackageManager packageManager = null;
		ApplicationInfo applicationInfo = null;
		try {
			packageManager = app.getApplicationContext().getPackageManager();
			applicationInfo = packageManager.getApplicationInfo(
					app.getPackageName(), 0);

		} catch (PackageManager.NameNotFoundException e) {
			applicationInfo = null;
		}
		String applicationName = (String) packageManager
				.getApplicationLabel(applicationInfo);
		return applicationName;
	}

	private static String getCurVersion() {
		try {
			PackageManager manager = app.getPackageManager();
			PackageInfo info = manager.getPackageInfo(app.getPackageName(), 0);
			String version = info.versionName;
			return version;
		} catch (Exception e) {
			e.printStackTrace();
			return "";
		}
	}

	private static String getDeviceInfo() {
		String verStr = NativeUtils.getInstallationUUID() + "_"
				+ android.os.Build.MODEL + "_"
				+ android.os.Build.VERSION.RELEASE;
		// + android.os.Build.VERSION.SDK + ","
		return verStr;
	}

	public static String getPackNameStr() {
		return app.getPackageName();
	}

	private static final String UMENG_CHANNEL = "UMENG_CHANNEL";

	public static String getChannelStr() {
		String str = "";
		PackageInfo packageInfo;
		ApplicationInfo appInfo;
		try {
			appInfo = app.getPackageManager().getApplicationInfo(
					app.getPackageName(), PackageManager.GET_META_DATA);
			str = appInfo.metaData.get(UMENG_CHANNEL).toString(); // getString(UMENG_CHANNEL);
			// Log.e(str,"hahatao");
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return str;
	}

	public static void showModalWebView(final String url) {
		app.runOnUiThread(new Runnable() {
			@Override
			public void run() {

				// Uri 使用. 打开网页
				Uri uri = Uri.parse(url);
				Intent it = new Intent(Intent.ACTION_VIEW, uri);
				app.startActivity(it);
			}
		});
	}

	public static void quitApp() {
		app.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				android.os.Process.killProcess(android.os.Process.myPid());
			}
		});
	}

	// 退出框:
	public static void showQuitAppDialog() {
		app.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				AlertDialog.Builder builder = null;
				builder = new AlertDialog.Builder(app);
				builder.setTitle(R.string.q_title);
				builder.setMessage(R.string.q_contente);
				builder.setIcon(R.drawable.icon);
				builder.setPositiveButton(R.string.q_ok,
						new DialogInterface.OnClickListener() {
							@Override
							public void onClick(DialogInterface dialog,
									int which) {
								// TODO Auto-generated method stub
								android.os.Process
										.killProcess(android.os.Process.myPid());
							}
						});
				builder.setNegativeButton(R.string.q_cancel, null);
				builder.setCancelable(true);
				builder.show();
			}
		});
	}

	// 常用对话框实现.
	public static void okAlert(final String title, final String message,
			final String ok, final int tag) {
		app.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				DialogInterface.OnClickListener listener = new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, final int which) {
						app.runOnGLThread(new Runnable() {
							@Override
							public void run() {
								try {
									JSONObject dic = new JSONObject();
									dic.put("buttonIdex", which);
									CPPJsNotiFun(String.format(
											"_alertMessage_%d", tag), dic
											.toString());
								} catch (JSONException ex) {
								}
							}
						});
					}
				};

				AlertDialog.Builder alertDialog = new AlertDialog.Builder(app)
						.setTitle(title).setMessage(message)
						.setCancelable(false).setIcon(R.drawable.icon)
						.setPositiveButton(ok, listener);
				alertDialog.create().show();
			}
		});
	}

	public static void askAlert(final String title, final String message,
			final String no, final String yes, final int tag) {
		app.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				DialogInterface.OnClickListener listener = new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, final int which) {
						app.runOnGLThread(new Runnable() {
							@Override
							public void run() {
								try {
									JSONObject dic = new JSONObject();
									dic.put("buttonIdex", which);
									CPPJsNotiFun(String.format(
											"_alertMessage_%d", tag), dic
											.toString());
								} catch (JSONException ex) {
								}
							}
						});
					}
				};

				AlertDialog.Builder alertDialog = new AlertDialog.Builder(app)
						.setTitle(title).setMessage(message)
						.setCancelable(false).setIcon(R.drawable.icon)
						.setPositiveButton(yes, listener)
						.setNegativeButton(no, listener);
				alertDialog.create().show();
			}
		});
	}

	public static void inputAlert(final String title, final String message,
			final String cancel, final String ok, final String content,
			final int tag) {
		app.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				// 输入对话框.
				final EditText et = new EditText(app);
				et.setText(content);
				// et.setTransformationMethod(PasswordTransformationMethod.getInstance());

				DialogInterface.OnClickListener listener = new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, final int which) {
						app.runOnGLThread(new Runnable() {
							@Override
							public void run() {
								try {
									JSONObject dic = new JSONObject();
									dic.put("buttonIdex", which);
									dic.put("content", et.getText().toString());
									CPPJsNotiFun(String.format(
											"_alertMessage_%d", tag), dic
											.toString());
								} catch (JSONException ex) {
								}
							}
						});
					}
				};

				AlertDialog.Builder alertDialog = new AlertDialog.Builder(app)
						.setTitle(title).setMessage(message)
						.setCancelable(false).setIcon(R.drawable.icon)
						.setView(et).setPositiveButton(ok, listener)
						.setNegativeButton(cancel, listener);
				alertDialog.create().show();
			}
		});
	}

	public static void btnsAlert(final String title, final String message,
			final String btnsSharpSepStr, final int choice, final int tag) {
		app.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				// 输入对话框.
				AlertDialog.Builder builder = new AlertDialog.Builder(app)
						.setTitle(title)
						.setIcon(android.R.drawable.ic_dialog_info)
						.setSingleChoiceItems(btnsSharpSepStr.split("#"),
								choice, new DialogInterface.OnClickListener() {
									public void onClick(DialogInterface dialog,
											final int which) {
										// Log.v("single select:",String.format("%d",which));
										dialog.dismiss();
										app.runOnGLThread(new Runnable() {
											@Override
											public void run() {
												try {
													JSONObject dic = new JSONObject();
													dic.put("buttonIdex", which);
													CPPJsNotiFun(String.format(
															"_alertMessage_%d",
															tag), dic
															.toString());
												} catch (JSONException ex) {
												}
											}
										});
									}
								}).setCancelable(false);
				// .setNegativeButton("取消", null);
				builder.show();
			}
		});
	}
	
	public static void toastMessage(final String msg) {
		app.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				Toast.makeText(app.getApplicationContext(), msg,
						Toast.LENGTH_SHORT).show();
			}
		});
	}
}
